function calcPlayerTrans(deck,strategy);
% Given the deck pdf and a strategy matrix for the player, 
% computes the start-finish transition matrices for the player's hand.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Blackjack Markov Chain Toolbox, v1.0 (08 Jan 2009)
%%% 
%%% Companion to manuscript 
%%%    "A Markov Chain Analysis of Blackjack Strategy"
%%%    by Michael B. Wakin and Christopher J. Rozell, 2004.
%%%    available at: http://inside.mines.edu/~mwakin/papers/mcbj.pdf
%%%
%%% Software Copyright 2009 by 
%%%    Michael B. Wakin, Colorado School of Mines, mwakin@mines.edu
%%%    Christopher J. Rozell, Georgia Tech, crozell@gatech.edu 
%%%
%%% This software is released under a Creative Commons license
%%% (Attribution 3.0 Unported): 
%%%     http://creativecommons.org/licenses/by/3.0/
%%% You are free:
%%%    * to Share  to copy, distribute and transmit the work
%%%    * to Remix  to adapt the work
%%% Under the following conditions:
%%%   * You must attribute the work by retaining this header information
%%%     crediting the original authors and linking to the original
%%%     manuscript.
%%%   * For any reuse or distribution, you must make clear to others the 
%%%     license terms of this work.
%%%   * Any of the above conditions can be waived if you get permission 
%%%     from the copyright holder.
%%%   * Nothing in this license impairs or restricts the author's 
%%%     moral rights.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


global PLAYERfirst PLAYERstartHard PLAYERmidHard PLAYERstartSoft PLAYERmidSoft PLAYERstartPair PLAYERterm PLAYERdoubTerm PLAYERtermSplit PLAYERblackjack PLAYERsurrender PLAYERbust PLAYERdoubBust

global houseRules
global playerMoves
global P Pinf

% Unpack structs for faster code
fields = fieldnames(houseRules);
for ii = 1:length(fields)
  name = fields{ii};
  eval([name '= houseRules.' name ';']);
end	
fields = fieldnames(playerMoves);
for ii = 1:length(fields)
  name = fields{ii};
  eval([name '= playerMoves.' name ';']);
end	
fields = fieldnames(strategy);
for ii = 1:length(fields)
  name = fields{ii};
  eval([name '= strategy.' name ';']);
end	

% Let strategy matrices be PAIR,HARD, and SOFT, 
%   indexed by row (pTot -- HARD 4 to 21 -- SOFT 12 to 21) 
%   and col (dCard -- 2 to 11). PAIR is indexed by row
%   (card -- 2 to 11) and col (dCard -- 2 to 11)

P(:,:,:) = 0;

% Set absorbing states
for ii = 4:21
  P(:,PLAYERterm(ii),PLAYERterm(ii)) = 1;
end
for ii = 6:21
  P(:,PLAYERdoubTerm(ii),PLAYERdoubTerm(ii)) = 1;
end
for ii = 2:11
  P(:,PLAYERtermSplit(ii),PLAYERtermSplit(ii)) = 1;
end
P(:,PLAYERblackjack,PLAYERblackjack) = 1;
P(:,PLAYERsurrender,PLAYERsurrender) = 1;
P(:,PLAYERbust,PLAYERbust) = 1;
P(:,PLAYERdoubBust,PLAYERdoubBust) = 1;

% Set first transitions
for ii = 2:11
  for pNew = 2:11
    if (ii ~= pNew)
      % Start with no pair
      newPtot = ii + pNew;
      newPsoft = ((ii == 11) | (pNew == 11));
      if (newPtot > 21)
        newPtot = newPtot - 10;
        newPsoft = 1;
      end
      if (newPtot == 21)
        P(:,PLAYERfirst(ii),PLAYERblackjack) = P(:,PLAYERfirst(ii),PLAYERblackjack) + deck(pNew);
      elseif (newPsoft)
        P(:,PLAYERfirst(ii),PLAYERstartSoft(newPtot)) = P(:,PLAYERfirst(ii),PLAYERstartSoft(newPtot)) + deck(pNew);
      else
        P(:,PLAYERfirst(ii),PLAYERstartHard(newPtot)) = P(:,PLAYERfirst(ii),PLAYERstartHard(newPtot)) + deck(pNew);
      end
    else
      % Start with pair
      P(:,PLAYERfirst(ii),PLAYERstartPair(ii)) = P(:,PLAYERfirst(ii),PLAYERstartPair(ii)) + deck(pNew);
    end
  end
end

% Set startHard transitions
for ii = 4:21
  for dCard = 2:11
    tableMove = HARD(ii-3,dCard-1);
    if ((tableMove == SR) & (~SRA))
      tableMove = H;
    end
    if (tableMove == SR)
      P(dCard,PLAYERstartHard(ii),PLAYERsurrender) = 1; 
    elseif (tableMove == S)
      P(dCard,PLAYERstartHard(ii),PLAYERterm(ii)) = 1;
    elseif (tableMove == H)
      for pNew = 2:11
        newPtot = ii + pNew;
        newPsoft = (pNew == 11);
        if ((newPtot > 21) & newPsoft)
          newPtot = newPtot - 10;
          newPsoft = 0;
        end
        if (newPtot > 21)
          P(dCard,PLAYERstartHard(ii),PLAYERbust) = P(dCard,PLAYERstartHard(ii),PLAYERbust) + deck(pNew);
        elseif (newPsoft)
          P(dCard,PLAYERstartHard(ii),PLAYERmidSoft(newPtot)) = P(dCard,PLAYERstartHard(ii),PLAYERmidSoft(newPtot)) + deck(pNew);
        else
          P(dCard,PLAYERstartHard(ii),PLAYERmidHard(newPtot)) = P(dCard,PLAYERstartHard(ii),PLAYERmidHard(newPtot)) + deck(pNew);
        end
      end
    elseif ((tableMove == DB) | (tableMove == DS))
      for pNew = 2:11
        newPtot = ii + pNew;
        newPsoft = (pNew == 11);
        if ((newPtot > 21) & newPsoft)
          newPtot = newPtot - 10;
          newPsoft = 0;
        end
        if (newPtot > 21)
          P(dCard,PLAYERstartHard(ii),PLAYERdoubBust) = P(dCard,PLAYERstartHard(ii),PLAYERdoubBust) + deck(pNew);
        else
          P(dCard,PLAYERstartHard(ii),PLAYERdoubTerm(newPtot)) = P(dCard,PLAYERstartHard(ii),PLAYERdoubTerm(newPtot)) + deck(pNew);
        end
      end
    else 
      fprintf('Unknown move !!! \n');
      tableMove
    end
  end
end

% Set midHard transitions
for ii = 5:21
  for dCard = 2:11
    tableMove = HARD(ii-3,dCard-1);
    if (tableMove == SR)
      tableMove = H;
    elseif (tableMove == DB)
      tableMove = H;
    elseif (tableMove == DS)
      tableMove = S;
    end
    if (tableMove == S)
      P(dCard,PLAYERmidHard(ii),PLAYERterm(ii)) = 1;
    elseif (tableMove == H)
      for pNew = 2:11
        newPtot = ii + pNew;
        newPsoft = (pNew == 11);
        if ((newPtot > 21) & newPsoft)
          newPtot = newPtot - 10;
          newPsoft = 0;
        end
        if (newPtot > 21)
          P(dCard,PLAYERmidHard(ii),PLAYERbust) = P(dCard,PLAYERmidHard(ii),PLAYERbust) + deck(pNew);
        elseif (newPsoft)
          P(dCard,PLAYERmidHard(ii),PLAYERmidSoft(newPtot)) = P(dCard,PLAYERmidHard(ii),PLAYERmidSoft(newPtot)) + deck(pNew);
        else
          P(dCard,PLAYERmidHard(ii),PLAYERmidHard(newPtot)) = P(dCard,PLAYERmidHard(ii),PLAYERmidHard(newPtot)) + deck(pNew);
        end
      end
    else 
      fprintf('Unknown move !!! \n');
      tableMove
    end
  end
end

% Set startSoft transitions
for ii = 12:21
  for dCard = 2:11
    tableMove = SOFT(ii-11,dCard-1);
    if ((tableMove == SR) & (~SRA))
      tableMove = H;
    end
    if (tableMove == SR)
      P(dCard,PLAYERstartSoft(ii),PLAYERsurrender) = 1; 
    elseif (tableMove == S)
      P(dCard,PLAYERstartSoft(ii),PLAYERterm(ii)) = 1;
    elseif (tableMove == H)
      for pNew = 2:11
        newPtot = ii + pNew;
        newPsoft = 1;
        if (newPtot > 21)
          newPtot = newPtot - 10;
          newPsoft = (pNew == 11);
        end
        if (newPsoft)
          P(dCard,PLAYERstartSoft(ii),PLAYERmidSoft(newPtot)) = P(dCard,PLAYERstartSoft(ii),PLAYERmidSoft(newPtot)) + deck(pNew);
        else
          P(dCard,PLAYERstartSoft(ii),PLAYERmidHard(newPtot)) = P(dCard,PLAYERstartSoft(ii),PLAYERmidHard(newPtot)) + deck(pNew);
        end
      end
    elseif ((tableMove == DB) | (tableMove == DS))
      for pNew = 2:11
        newPtot = ii + pNew;
        newPsoft = 1;
        if (newPtot > 21)
          newPtot = newPtot - 10;
          newPsoft = (pNew == 11);
        end
        P(dCard,PLAYERstartSoft(ii),PLAYERdoubTerm(newPtot)) = P(dCard,PLAYERstartSoft(ii),PLAYERdoubTerm(newPtot)) + deck(pNew);
      end
    else 
      fprintf('Unknown move !!! \n');
      tableMove
    end
  end
end

% Set midSoft transitions
for ii = 13:21
  for dCard = 2:11
    tableMove = SOFT(ii-11,dCard-1);
    if (tableMove == SR)
      tableMove = H;
    elseif (tableMove == DB)
      tableMove = H;
    elseif (tableMove == DS)
      tableMove = S;
    end
    if (tableMove == S)
      P(dCard,PLAYERmidSoft(ii),PLAYERterm(ii)) = 1;
    elseif (tableMove == H)
      for pNew = 2:11
        newPtot = ii + pNew;
        newPsoft = 1;
        if (newPtot > 21)
          newPtot = newPtot - 10;
          newPsoft = (pNew == 11);
        end
        if (newPsoft)
          P(dCard,PLAYERmidSoft(ii),PLAYERmidSoft(newPtot)) = P(dCard,PLAYERmidSoft(ii),PLAYERmidSoft(newPtot)) + deck(pNew);
        else
          P(dCard,PLAYERmidSoft(ii),PLAYERmidHard(newPtot)) = P(dCard,PLAYERmidSoft(ii),PLAYERmidHard(newPtot)) + deck(pNew);
        end
      end
    else 
      fprintf('Unknown move !!! \n');
      tableMove
    end
  end
end

% Set startPair transitions 
for ii = 2:11
  if (ii == 11)
    pTot = 12;
    pSoft = 1;
  else
    pTot = 2*ii;
    pSoft = 0;
  end
  for dCard = 2:11
    tableMove = PAIR(ii-1,dCard-1);    
    if ((tableMove == SR) & (~SRA))
      tableMove = H;
    end
    if (tableMove == SR)
      P(dCard,PLAYERstartPair(ii),PLAYERsurrender) = 1; 
    elseif (tableMove == S)
      P(dCard,PLAYERstartPair(ii),PLAYERterm(pTot)) = 1;
    elseif (tableMove == SP)
      P(dCard,PLAYERstartPair(ii),PLAYERtermSplit(ii)) = 1;
    elseif (tableMove == H)
      for pNew = 2:11
        newPtot = pTot + pNew;
        newPsoft = (pSoft | (pNew == 11));
        if ((newPtot > 21) & newPsoft)
          newPtot = newPtot - 10;
          newPsoft = (pSoft & (pNew == 11));  %%% MBW: Changed 6/15/04 (was zero)
        end
        if (newPtot > 21)
          P(dCard,PLAYERstartPair(ii),PLAYERbust) = P(dCard,PLAYERstartPair(ii),PLAYERbust) + deck(pNew);
        elseif (newPsoft)
          P(dCard,PLAYERstartPair(ii),PLAYERmidSoft(newPtot)) = P(dCard,PLAYERstartPair(ii),PLAYERmidSoft(newPtot)) + deck(pNew);
        else
          P(dCard,PLAYERstartPair(ii),PLAYERmidHard(newPtot)) = P(dCard,PLAYERstartPair(ii),PLAYERmidHard(newPtot)) + deck(pNew);
        end
      end
    elseif ((tableMove == DB) | (tableMove == DS))
      for pNew = 2:11
        newPtot = pTot + pNew;
        newPsoft = (pSoft | (pNew == 11));
        if ((newPtot > 21) & newPsoft)
          newPtot = newPtot - 10;
          newPsoft = (pSoft & (pNew == 11));  %%% MBW: Changed 6/15/04 (was zero)
        end
        if (newPtot > 21)
          P(dCard,PLAYERstartPair(ii),PLAYERdoubBust) = P(dCard,PLAYERstartPair(ii),PLAYERdoubBust) + deck(pNew);
        else
          P(dCard,PLAYERstartPair(ii),PLAYERdoubTerm(newPtot)) = P(dCard,PLAYERstartPair(ii),PLAYERdoubTerm(newPtot)) + deck(pNew);
        end
      end
    else 
      fprintf('Unknown move !!! \n');
      tableMove
    end    
  end
end

% Compute player's "infinite" matrix, with all probability in the absorbing states
Pinf(:,:,:) = 0;
for ii = 2:11
  Pinf(ii,:,:) = squeeze(P(ii,:,:))^18;
end
